# Sample Python code to generate Hamming Distance macro
# Kevin Angstadt
# angstadt {at} virginia.edu

from micronap.sdk import *

def main():
    A = Anml()
    M = A.CreateMacroDef(anmlId='hamming')
    
    # the number of mismatches
    d = 3
    
    # length of string to compare against
    string_length = 7
    
    ste_list = dict()
    
    #set up param ste lists
    positive_param = []
    negative_param = []
    
    for i in range(string_length):
        positive_param.append([])
        negative_param.append([])
    
    # generate automata structure
    for i in range(2*d+1):
        for j in range(i/2,string_length):
            ste_id = '_'+str(i)+'_'+str(j)
            
            # set default values
            # we will replace these when instantiating
            if i%2 == 0:
                pattern = 'a'
            else:
                pattern = '[^a]'
            
            # starting states 
            if j == 0:
                start = AnmlDefs.ALL_INPUT
            else:
                start = AnmlDefs.NO_START
            
            # if we reach the end, report
            if j == string_length-1:
                report = True
            else:
                report = False
                
            ste_list[ste_id] = M.AddSTE(pattern,anmlId=ste_id,startType=start,match=report)
            
            # connect sequences of matching characters
            if i%2 == 0 and j>i/2:
                old_id = '_'+str(i)+'_'+str(j-1)
                M.AddAnmlEdge(ste_list[old_id], ste_list[ste_id])
            
            # diagonal mismatch transitions
            if i>0 and j>(i-1)/2:
                old_id = '_'+str(i-1)+'_'+str(j-1)
                M.AddAnmlEdge(ste_list[old_id], ste_list[ste_id])
            
            # two mismatches in a row
            if i>1 and i%2 == 1:
                old_id = '_'+str(i-2)+'_'+str(j-1)
                M.AddAnmlEdge(ste_list[old_id], ste_list[ste_id])
                
            # add ste to parameter list
            if i%2 == 0:
                positive_param[j].append(ste_list[ste_id])
            else:
                negative_param[j].append(ste_list[ste_id])
                
    # set parameter information
    for i in range(string_length):
        M.AddMacroParam(paramName='%'+str(i+1),elementRefs=positive_param[i])
        M.AddMacroParam(paramName='%n'+str(i+1),elementRefs=negative_param[i])
    
    M.ExportAnml('hamming'+str(string_length)+'_'+str(d)+'.anml')

if __name__ == '__main__':
    main()